# -*- coding: utf-8 -*-
#
# Copyright (c) 2018, HILSTER GmbH, Germany - https://hilster.de
# All rights reserved.
#
# This file was generated by wrappergenerator.py
#

from __future__ import absolute_import, division, print_function, unicode_literals

from oser._array_type import Array as _Array


class Array(_Array):
    """
    ``Array`` builds an array serializer with a fixed or variable length.
    """

    def __init__(self, length, prototype, args=None, kwargs=None, values=None):
        """
        Args:
            length (callable or integer): states the array length. Can be a callable (e.g. lambda) or a scalar.
            prototype (an OSER class): a class that is the prototype for `values`
            args=None (tuple): a tuple of positional arguments that are passed to `prototype` when it is instantiated when
                array size is extended.
                Note: don't forget a leading comma when passing one value ``(1,)`` (``(1)`` is an integer!).
            kwargs=None (dict): a dictionary of named arguments that are passed to `prototype` when it is instantiated when
                array size is extended.
            values=None (list containing instances of ``prototype``): the initial array values. Can be left empty. If empty the array is automatically
                resized with instances of ``prototype(*args, **kwargs)``.
        """
        return super(Array, self).__init__(length=length, prototype=prototype, args=args, kwargs=kwargs, values=values)

    def byte_size(self):
        """
        Return the current size in bytes.
        """
        return super(Array, self).byte_size()

    def decode(self, data, full_data=b'', context_data=b''):
        """
        Decode a binary string and return the number
        of bytes that were decoded.

        Args:
            data (bytes): the data buffer that is decoded.
            full_data (bytes): the binary data string until the part to be decoded. The user normally does not need to supply this.
            context_data (bytes): the binary data of the current context. The user normally does not need to supply this.

        Returns:
            int: the number of bytes that were decoded.
        """
        return super(Array, self).decode(data=data, full_data=full_data, context_data=context_data)

    def encode(self, full_data=b'', context_data=b''):
        """
        Return the encoded binary string.

        Args:
            full_data (bytes): the binary data string until the part to be
                encoded. The user normally does not need to supply this.
            context_data (bytes): the binary data of the current context.
                The user normally does not need to supply this.

        Returns:
            bytes: the encoded binary string.
        """
        return super(Array, self).encode(full_data=full_data, context_data=context_data)

    def fuzzing_iterator(self, copy=False):
        """
        The fuzzing iterator iterates over all combinations
        of set fuzzing values (:func:`oser.ByteType.set_fuzzing_values`).
        If no fuzzing values are set the current struct is yielded.

        Args:
            copy=False (bool): if set to ``True`` the generated
                fuzzing values are deep copies of the original.
                Creating these deep copies is slow. If set to ``False`` the 
                original struct is retruned and the generated value must be
                used immediately since the next generated overwrites the values
                on the same value.

        Yields:
            the fuzzing combinations.
        """
        return super(Array, self).fuzzing_iterator(copy=copy)

    def get(self):
        """
        Return the value.

        Returns:
            the value
        """
        return super(Array, self).get()

    def introspect(self, stop_at=None):
        """
        Return the introspection representation of the object
        as a string.

        Args:
            stop_at=None (object): stop introspection at ``stop_at``.
        """
        return super(Array, self).introspect(stop_at=stop_at)

    def root(self):
        """
        return root element
        """
        return super(Array, self).root()

    def size(self):
        """
        Return the current size in bytes or bits according to the context.
        """
        return super(Array, self).size()

    def up(self):
        """
        return parent element
        """
        return super(Array, self).up()

