# -*- coding: utf-8 -*-
#
# Copyright (c) 2018, HILSTER GmbH, Germany - https://hilster.de
# All rights reserved.
#

from __future__ import absolute_import, division, print_function, unicode_literals

from .core import ByteType


class Nothing(ByteType):
    """
    ``Nothing`` does not appear in the encoded data and not in the string or
    intorspection.
    """

    _struct_representation = ""
    _byte_count = 0

    def __init__(self):
        super(Nothing, self).__init__()

    def encode(self, full_data=b"", context_data=b""):
        """
        Return the encoded binary string. This is always an empty string.

        Args:
            full_data (bytes): the binary data string until the part to be
                encoded. The user normally does not need to supply this.
            context_data (bytes): the binary data of the current context.
                The user normally does not need to supply this.

        Returns:
            bytes: the encoded binary string.
        """
        return b""

    def decode(self, data, full_data=b"", context_data=b""):
        """
        Decode a binary string and return the number
        of bytes that were decoded.

        Args:
            data (bytes): the data buffer that is decoded.
            full_data (bytes): the binary data string until the part to be
                decoded. The user normally does not need to supply this.
            context_data (bytes): the binary data of the current context.
                The user normally does not need to supply this.

        Returns:
            int: the number of bytes that were decoded.
        """
        return 0

    def get(self):
        """
        Always returns ``None``.

        Returns:
            None
        """
        return None

    def set(self, value):
        """
        Set the value. Always raises an Exception.

        Args:
            value: the new value

        """
        raise Exception("Nothing's value cannot be set")

    def __str__(self, indent=0, name=None, stop_at=None):
        return ""

    def _introspect(self, byte_index, bit_index, indent=0, name=None, stop_at=None):
        ret = ""

        return ret, byte_index, bit_index
